import * as WebSocket from 'ws';
import ClientManager from '../clientManager';
import ChannelManager from '../channelManager';
import ChannelBase from '../channels/channelBase';
import StudentProgressMonitorChannel from '../channels/services/studentProgressMonitorChannel';

var messageManager = require('../messageManager');
var logger = require('../logger');

class ClientBase {
  ws: WebSocket;
  data: any;
  id: number;
  channel: ChannelBase|StudentProgressMonitorChannel|null;
  clientManager: ClientManager;
  channelManager: ChannelManager;

  constructor(data: any, ws: WebSocket, channelManager: ChannelManager, clientManager: ClientManager) {
    this.ws = ws;
    this.data = data;
    this.id = data.user_id;
    this.channel = null;
    this.clientManager = clientManager;
    this.channelManager = channelManager;
  }

  getData() {
    return this.data;
  }

  type() {
    return this.data.client_type;
  }

  clientType() {
    return this.data.client;
  }

  connectToChannel(channel: StudentProgressMonitorChannel) {
    this.channel = channel;

    var messageListener = (message: any) => {
      logger.accessLog.info(`starting message transaction on channel ${channel.id}: `, {message: message});
      message = messageManager.prepareMessage(message);

      if (!message.error) {
        if (message['message_type'] == 'broadcast') {
          channel.broadcastMessage(this, message);
        } else if (message['message_type'] == 'changeChannel') {
          this.ws.removeListener('message', messageListener);
          this.channelManager.changeChannel(this, message);
        }

        logger.accessLog.info(`message transaction complete on channel ${channel.id}: `, {message: message});
      } else {
        logger.errorLog.info(`Validation failed, please review schema: ${channel.id}`, {data: {message: message, error: message.error}});
      }
    }

    this.ws.on('message', messageListener);

    this.ws.on('close', (reasonCode: string, description: string) => {
      logger.accessLog.info(`closing connection for client ${this.id}`);

      if (this.channel) {
        this.channel.removeClient(this.id);
      }

      this.clientManager.removeClient(this.id);
      logger.accessLog.info(`closed connection for client ${this.id}`);
    });
  }

  replaceWebSocket(ws: WebSocket) {
    this.ws.close();
    this.ws = ws;
  }
};

export default ClientBase;
