//external imports
import * as express from 'express';
import * as https from 'https';
import * as WebSocket from 'ws';
import * as fs from 'fs';
import * as jwt from 'jsonwebtoken';
import * as url from 'url';

//internal imports
var routes = require('./routes');
var app = require('./config/app');
var logger = require('./logger');
import ClientManager from './clientManager';
import ChannelManager from './channelManager';
import ClientBase from './clients/clientBase';
import ServicesClient from './clients/servicesClient';


// if (app.environment == 'development') {
//   var privateKey  = fs.readFileSync(app.privateKey, 'utf8');
//   var certificate = fs.readFileSync(app.certificate, 'utf8');
//   var options = {key: privateKey, cert: certificate, hostname: app.hostname};

//   const application = express();
//   const server = https.createServer(options, application);
//   var wss = new WebSocket.Server({ server: server, maxPayload:250000, host: app.hostname });

//   application.use(express.json());
//   application.use('', routes);

//   server.listen(app.port, () => {
//     console.log(`Braid v${app.version} is running!\n`);
//     logger.accessLog.info(`Braid v${app.version} is running!\n`);
//   });
// } else {
  var wss = new WebSocket.Server({ maxPayload:250000, port: app.port });
// }

let clientManager = new ClientManager();
let channelManager = new ChannelManager();

function connectionManager() {
  wss.on('connection', (ws: WebSocket, request: any, args: string) => {
    var result = JSON.parse(validateJWT(request));

    if (result.error) {
      ws.send(`Unable to validate JWT, please try again or contact support...`);
      ws.close();
    } else {
      var data = result.data;
      logger.accessLog.info(`Client Connected: ${data.user_id}`);

      if (!channelManager.channelExists(data.channel)) {
        channelManager.createChannel(data);
      }

      if (clientManager.clientExists(data.user_id)) {
        var client: ClientBase|ServicesClient|null = clientManager.getClient(data.user_id);

        if (client != null) {
          client.replaceWebSocket(ws);
        }
      } else {
        var client: ClientBase|ServicesClient|null = clientManager.addClient(data, channelManager, ws);
      }

      if (client != null) {
        channelManager.addClientToChannel(client, data.channel);
      }

      //remove any channels that have no users in them
      logger.accessLog.info(`Purging empty channels...`);
      channelManager.purgeEmptyChannels();

      ws.send(`Hi there, welcome to braid, Measures Web Socket server. Connecting all our services!\nYou are currently in channel: ${data.channel}`);
    }
  });
}

function validateJWT(request: any) {
  try {
    var query = url.parse(request.url, true).query
    var token = query.token || (app.environment == 'development' ? app.devToken : '');
    return JSON.stringify(jwt.verify(token, app.secret, app.signOptions));
  } catch (e) {
    console.log(e);
    return JSON.stringify({error: e});
  }
}

function startServer() {
  connectionManager();
}

startServer();

module.exports = {
  clientManager: clientManager,
  channelManager: channelManager,
  connectionManager: connectionManager
}
